import * as React from 'react';
import {ButtonField} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {makeStyles} from "@material-ui/core/styles";
import {yellow} from "@material-ui/core/colors";
import {useSnackbar} from "notistack";
import {RxModuleComponentModel} from "../../stores/RxModuleComponentStore";

const useStyles = makeStyles({
    root: {
        backgroundColor: yellow[500],
        color: "black"
    }
});

type Props = {
    model: RxModuleComponentModel
}

export const RestartButton: React.FunctionComponent<Props> = ({model}) => {
    const classes = useStyles();
    const [loading, setLoading] = React.useState(false);
    const {enqueueSnackbar} = useSnackbar();

    const disabled = !model.reachable;

    const handleClick = React.useCallback(async () => {
        setLoading(true);

        try {
            await model.restart();
            enqueueSnackbar(t`Restart requested`, {variant: "success"});
        } catch {
            enqueueSnackbar(t`Couldn't request restart`, {variant: "error"});
        }

        setLoading(false);
    }, [enqueueSnackbar, model]);

    return (
        <ButtonField
            disabled={disabled}
            classes={classes}
            onClick={handleClick}
            text={t`Restart`}
            loading={loading}
        />
    )
};