# == Schema Information
#
# Table name: shifts
#
#  id         :bigint           not null, primary key
#  color      :string(4000)
#  name       :string(4000)
#  created_at :datetime         not null
#  updated_at :datetime         not null
#
class Shift < ApplicationRecord
  has_many :shift_schedules

  class << self
    def use_shifts?
      # todo read this from the licence
      if ENV.has_key? "USE_SHIFTS"
        ENV["USE_SHIFTS"]&.downcase == "true"
      else
        RxModuleComponent.where(module_id: "rx_data").exists?
      end
    end

    def union_range(a, b)
      ([a.begin, b.begin].compact.min)..([a.end, b.end].compact.max)
    end

    def intersect_range(a, b)
      return nil unless a.overlaps?(b)
      ([a.begin, b.begin].compact.max)..([a.end, b.end].compact.min)
    end

    def merge_overlapping_ranges(overlapping_ranges)
      overlapping_ranges.sort_by(&:begin).inject([]) do |ranges, range|
        if !ranges.empty? && ranges.last.overlaps?(range)
          ranges[0...-1] + [union_range(ranges.last, range)]
        else
          ranges + [range]
        end
      end
    end

    def active_ranges_between(start_date, end_date)
      raise "Start and End Date are required to generate a schedule sequence" unless start_date && end_date
      capping_range = start_date..end_date
      merge_overlapping_ranges(instances_for(start_date, end_date).reduce([]) do |bucket, i|
        bucket.concat(i[:dates].map do |d|
          intersect_range(capping_range, d[:start]..d[:end])
        end.reject do |r|
          r.begin == r.end
        end)
      end)
    end

    def instances_for(start_date, end_date, opts = {})
      raise "Start and End Date are required to generate a schedule sequence" unless start_date && end_date # otherwise this method will run until we run out of memory
      if start_date > end_date
        tmp = start_date
        start_date = end_date
        end_date = tmp
      end
      start_arel = ShiftSchedule.arel_table[:start_date]
      end_arel = ShiftSchedule.arel_table[:end_date]
      query = ShiftSchedule.where(start_arel.lteq(end_date)).where(end_arel.gteq(start_date).or(end_arel.eq(nil))).eager_load(:shift)
      query = query.where(shift_id: opts[:shift_id]) if opts[:shift_id]
      query.all.map do |schedule|
        {
          schedule: schedule,
          dates: schedule.occurrences_between(start_date, end_date).map do |occurrence|
            {
              start: occurrence.start_time,
              end: occurrence.end_time
            }
          end
        }
      end.select do |obj|
        obj[:dates].length > 0
      end.each_with_object([]) do |current, bucket|
        schedule = current[:schedule]
        shift = schedule.shift
        dates = current[:dates]
        existing = bucket.find do |candidate|
          candidate[:shift] == shift
        end

        if existing
          existing[:dates].concat(dates)
        else
          bucket.push({
            shift: shift,
            dates: dates
          })
        end
      end
    end
  end

  def to_json(*args)
    JSON.generate(JSON.parse(super.as_json(*args)).merge(JSON.parse(render_jbuilder_partial)))
  end
end
